# -*- coding: utf8 -*-

from logging import getLogger
from base import BaseHandler
from thrift.transport import TTransport
from thrift.transport.TSocket import TSocket
from thrift.protocol import TBinaryProtocol
from artixbonusclient import AccountingLogicService
from artixbonusclient import ttypes
from decimal import Decimal
from contextlib import closing
import json

log = getLogger("su.artix.queue")


class ArtixBonusHandler(BaseHandler):
    """
    Обработчик сообщений для Артикс.Бонус
    """

    def __init__(self, dao, stopThreadsFlag, lastFailsLock, lastFails):
        super(ArtixBonusHandler, self).__init__(dao, stopThreadsFlag, lastFailsLock, lastFails)
        self.messageType = "artixbonus"
        self.name = "ArtixBonusHandler"
        self.socket = TSocket()
        self.transport = TTransport.TBufferedTransport(self.socket)
        self.client = AccountingLogicService.Client(TBinaryProtocol.TBinaryProtocol(self.transport))
        self.socket.setTimeout(180000)

    def sendMessage(self, message):
        try:
            log.info(u'Обработка сообщения %s: %s' % (message.messageid, message.body.replace('"','')))
            jsonMessage = json.loads(message.body)
            hosts = map(lambda x: x.split(':'), jsonMessage['hosts'])
            if not hosts:
                log.warn(u"Невозможно отправить сообщение %s, список серверов пуст" % message.messageid)
                return False
            for client in self.tryHosts(hosts):
                operation = self.operationChoice(client, jsonMessage['method'])

                if operation:
                    arguments = self.getArguments(jsonMessage)
                    result = operation(*arguments)
                    log.info(u'Метод выполнен, результат: %s' % result)
                    try:
                        log.info(result.description.decode("utf-8"))
                    except Exception, e:
                        pass

                    if result.success:
                        self.deleteSentMessage(message)
                        return True
                    return False

        except Exception as e:
            log.info(u"Ошибка при выполнеии метода: %s" % e.message)
        return False


    def send(self, message):
        if message.body == "DoNotSend":
            return False
        else:
            return True

    def tryHosts(self, hosts):
        for addr, port in hosts:
            log.debug(u'Проверка доступности %s:%s' % (addr, port))
            try:
                self.socket.host = addr
                self.socket.port = int(port)
                with closing(self.socket):
                    self.socket.open()
                    yield self.client
            except Exception, e:
                log.error(u'Ошибка: %s' % e)
                continue

    def operationChoice(self, client, method):
        if method == "add":
            operation = client.add
            log.info(u"Начисление бонусных баллов")
        elif method == "sub":
            operation = client.sub
            log.info(u"Списание бонусных баллов")
        elif method == "back":
            operation = client.back
            log.info(u"Отмена списания бонусных баллов")
        elif method == "commit":
            operation = client.commitTransaction
            log.info(u"Подтверждение транзакции")
        elif method == "rollback":
            operation = client.rollbackTransaction
            log.info(u"Отмена транзакции")
        elif method == "setAdditionalCardStatus":
            operation = client.setAdditionalCardStatus
            log.info(u"Установка дополнительного статуса карты")
        elif method == 'revertEarnBySessionId':
            operation = client.revertEarnBySessionId
            log.info(u'Отменяем начисленные бонусы')
        else:
            operation = None
            log.info(u"Невозможно выполнить метод %s" % method)

        return operation

    def getAdditionalAssetInfo(self, message):
        assetInfo = None
        if u'additionalInfo' in message:
            assetInfo = AccountingLogicService.AdditionalAssetInfo(
                dateFromCash = long(message['additionalInfo']['date_cash'])*1000,
                cashCode = message['additionalInfo']['cashCode'],
                shiftNumber = message['additionalInfo']['shiftNumber'],
                checkNumber = message['additionalInfo']['checkNumber']
                )
        return assetInfo

    def getArguments(self, jsonMessage):
        args = list()
        args.append(jsonMessage['terminalId'])

        if jsonMessage['method'] == 'setAdditionalCardStatus':
            card = AccountingLogicService.Card(cardNumber=str(jsonMessage['card']['cardNumber']),
                                               accountNumber=str(jsonMessage['card']['accountNumber']))

            args.append(card)
            args.append(jsonMessage['additionalStatus'].encode("utf8"))
        elif jsonMessage['method'] == 'revertEarnBySessionId':
            args.append(jsonMessage['cardNumber'])
            args.append(jsonMessage['sessionId'])
            args.append(jsonMessage['parentSessionId'])
            args.append(Decimal(jsonMessage['amount']) * Decimal(100))
            args.append(self.getAdditionalAssetInfo(jsonMessage))
        elif jsonMessage['method'] == 'rollback':
            args.append(jsonMessage['sessionId'])
            args.append(jsonMessage['cardNumber'])
        elif jsonMessage['method'] != 'commit':
            args.append(jsonMessage['sessionId'])
            args.append(jsonMessage['cardNumber'])
            args.append(Decimal(jsonMessage['amount']) * Decimal(100))
            args.append(self.getAdditionalAssetInfo(jsonMessage))
            assetGroup = None

            if 'assetGroup' in jsonMessage:
                assetGroup = AccountingLogicService.AssetGroup(
                    groupId=str(jsonMessage['assetGroup']['groupId']),
                    beginDate=long(jsonMessage['assetGroup']['data_beg']) * 1000,
                    endDate=long(jsonMessage['assetGroup']['data_end']) * 1000,
                    weight=int(jsonMessage['assetGroup']['weight'])
                )
            args.append(assetGroup)
            log.debug(
                u"терминал: %s, сессия: %s, карта: %s, сумма (коп): %d" % (args[0], args[1], args[2], args[3]))
        else:

            args.append(jsonMessage['sessionId'])
            args.append(jsonMessage['cardNumber'])
            dictStatus = ttypes.CardStatus._NAMES_TO_VALUES

            cardStatusStr = jsonMessage[u'cardStatus'] if u'cardStatus' in jsonMessage else None
            cardStatus = dictStatus[cardStatusStr] if cardStatusStr in dictStatus else None

            args.append(cardStatus)

            additionalCardStatus = None

            if 'additionalCardStatus' in jsonMessage:
                additionalCardStatus = jsonMessage['additionalCardStatus'].encode("utf8")

            args.append(additionalCardStatus)

            log.debug(
                u"терминал: %s, сессия: %s, карта: %s, новый статус карты: %s, новый дополнительный статус %s"
                % (args[0], args[1], args[2], cardStatusStr, additionalCardStatus.decode("utf-8")))

        return args
