# -*- coding: utf8 -*-

import signal, os, sys, ConfigParser
from handlers.abmloyalty import AbmLoyaltyHandler
from handlers.dummy import DummyHandler
from handlers.cardlab import CardLabHandler
from handlers.rsloyalty import RSLoyaltyHandler
from handlers.artixbonus import ArtixBonusHandler
from handlers.eps import EpsHandler
from handlers.excisemark import ExciseMarkHandler
from handlers.loymax import LoymaxHandler
from handlers.goldcrown import GoldCrownHandler
from handlers.spasibo import SpasiboHandler
from handlers.manzana import ManzanaHandler
from handlers.waybillnumber import WaybillNumberHandler
from serviceLock import ServiceLock
from db.dao import Dao
from logging import getLogger
from threading import Event
from threading import Lock
from time import sleep

log = getLogger('su.artix.queue')

class Queue(object):
    """
    Queue
    Класс обработки сообщений из БД очереди.
    Для каждого сообщения в потоке запускается обработчик для его обработки в зависимости от типа.
    Если отправка прошла успешно, обработчик обрабатывает следующее сообщение своего типа из базы данных.
    Если отправка прошла неудачно, обработчик завершает свою работу.
    Если получена команда на остановку сервиса, обработчики завершают обработку сообщений и завершают свою работу.
    """

    def __init__(self):
        self.serviceLock = ServiceLock('/linuxcash/cash/queue/pidfile', log)

        if self.serviceLock.lock(True):
            log.info(u'Запуск обработчика очередей')
        else:
            exit(0)

        self.dao = Dao()
        self.stopThreadsFlag = Event()
        self.stopThreadsFlag.clear()
        self.lastFailsLock = Lock()
        self.lastFails = dict()
        self.threads = dict()
        self.typesHandlers = dict()
        self.typesHandlers["abmloyalty"] = AbmLoyaltyHandler
        self.typesHandlers["dummy"] = DummyHandler
        self.typesHandlers["cardlab"] = CardLabHandler
        self.typesHandlers["rsloyalty"] = RSLoyaltyHandler
        self.typesHandlers["artixbonus"] = ArtixBonusHandler
        self.typesHandlers["eps"] = EpsHandler
        self.typesHandlers["excisemark"] = ExciseMarkHandler
        self.typesHandlers["loymax"] = LoymaxHandler
        self.typesHandlers["goldcrown"] = GoldCrownHandler
        self.typesHandlers["spasibo"] = SpasiboHandler
        self.typesHandlers["manzana"] = ManzanaHandler
        self.typesHandlers["waybillnumber"] = WaybillNumberHandler
        self.setSignalsHandler()

        cp = ConfigParser.ConfigParser()
        cp.read('config/queue.ini')

        try:
            self.dbWatcherTimeout = cp.getint('queue', 'timeout')
        except ValueError:
            self.dbWatcherTimeout = 20

    def signalTermHandler(self, signum, frame):
        log.info(u'Обработчик очередей получил сигнал завершения работы')
        self.stopThreadsFlag.set()

        while True:
            hasAliveThread = False
            for thread in self.threads.values():
                if thread.isAlive():
                    hasAliveThread = True
                    break
            if not hasAliveThread:
                break

        self.threads.clear()
        self.serviceLock.unlock()
        sys.exit(0)

    def setSignalsHandler(self):
        """
        Установка обработчика прерываний
        Обработчик должен сообщить дочерним потокам, что необходимо завершить работу.
        """
        signal.signal(signal.SIGTERM, self.signalTermHandler)
        signal.signal(signal.SIGINT, self.signalTermHandler)
        signal.signal(signal.SIGTSTP, self.signalTermHandler)
        signal.signal(signal.SIGHUP, self.signalTermHandler)
        signal.signal(signal.SIGSEGV, self.signalTermHandler)
        signal.signal(signal.SIGPIPE, self.signalTermHandler)

    def dbWatcher(self):
        """
        Функция мониторинга базы данных на наличие сообщения для обработчика
        Запускает обработчики в потоках
        """
        while not self.stopThreadsFlag.isSet():
            log.debug(u"Перезапуск обработчиков сообщений")
            for messageType in self.typesHandlers:
                if messageType not in self.threads:
                    self.startHandler(messageType)
                elif not self.threads[messageType].isAlive():
                    self.threads.pop(messageType)
                    self.startHandler(messageType)
            sleep(self.dbWatcherTimeout)

    def startHandler(self, messageType):
        """
        Запуск обработчика для обработки сообщений типа messageType
        """
        if self.dao.hasMessage(self.dao.Session, messageType):
            log.info(u'Запуск обработчика для обработки сообщений типа %s' % messageType)
            thread = self.typesHandlers[messageType](self.dao, self.stopThreadsFlag, self.lastFailsLock, self.lastFails)
            thread.start()
            self.threads[messageType] = thread
